/**
 * @name laravel-elixir-todo
 * @author Impbob
 *
 * @desc Generate TODO reports
 *
 * {@link https://github.com/pgilad/gulp-todo#usage link}
 *
 * @requires gulp-todo
 */

/*global require */

//  no-mixed-requires
var fs   = require('fs'),
    path = require('path');

// Additional package variables
var del        = require('del'),
    Elixir     = require('laravel-elixir'),
    gulp       = require('gulp'),
    gulpIf     = require('gulp-if'),
    template   = require('lodash.template'),
    through    = require('through2'),
    todo       = require('gulp-todo'),
    vinylPaths = require('vinyl-paths');

// Elixir dependent variables
var config = Elixir.config,
    notify = new Elixir.Notification(),
    Task   = Elixir.Task;

/**
 * Extend Elixir to include gulp-todo
 */
Elixir.extend('todo', function initElixirTodo (src, options) {
    'use strict';

    var paths   = new Elixir.GulpPaths()
        .src(src || [
                './*.{js,php}',
                config.appPath + '/**/*.php',
                config.get('assets.js.folder') + '/**/*.js',
                config.get('assets.css.folder') + '/**/*.css',
                config.get('assets.css.less.folder') + '/**/*.less',
                config.get('assets.css.sass.folder') + '/**/*.s?ss',
                config.testing.phpUnit.path + '/**/*.php',
            ]),
        onError = function onError (err) {
            notify.error(err, 'TODO Failed');
            this.emit('end');
        };

    // Check defaults
    options = options || [];
    if ('reporter' in options) {
        options.reporter = 'json';
    }



    // TASK:        Create TO DO file
    new Task('todo', function initTaskTodo () {                     //eslint-disable-line no-new
        return gulp.src(paths.src.path)
            .pipe(todo(options))
            .pipe(gulpIf(
                function checkReporter () {
                    return Boolean(options.reporterOptions);
                }, todo.reporter(options.reporter, options.reporterOptions), false))
            .pipe(gulpIf(function checkTodoCount (file) {
                return file.todos && Boolean(file.todos.length);
            }, gulp.dest('./'), vinylPaths(del)))
            .on('error', onError);
    });

    // TASK:        Append Template
    new Task('todo-inject', function initTaskTodo () {                     //eslint-disable-line no-new
        return gulp.src(paths.src.path)
            .pipe(todo(options))
            .pipe(gulpIf(
                function checkReporter () {
                    return Boolean(options.reporterOptions);
                }, todo.reporter(options.reporter, options.reporterOptions), false))
            .pipe(through.obj(function passThrough (file, enc, cb) {
                // Read file and replace '<%= marker %>' marker
                var tmpl = fs.readFileSync(options.template, 'utf8');
                var compiledTpl = template(tmpl);

                // SET:     Output
                var newContents = compiledTpl({
                    marker: file.todos.length ? file.contents.toString() : '### TODOs \nNo TODOs',
                });

                // UPDATE:         Filename
                file.path = path.join(file.base, options.output || 'readme.md');
                // UPDATE:         File
                file.contents = new Buffer(newContents);
                this.push(file);

                cb();
            }))
            .pipe(gulp.dest('./'))
            .on('error', onError);
    });
});
